/*******************************************************************************
 * Copyright (c) 2008 Plant Bioinformatics Group, IPK Gatersleben
 *******************************************************************************/
package de.ipk_gatersleben.ag_pbi.fluxbalance.visualization;

import java.awt.Color;
import java.util.Collection;
import java.util.TreeSet;

import org.AttributeHelper;
import org.ErrorMsg;
import org.graffiti.editor.MainFrame;
import org.graffiti.editor.MessageType;
import org.graffiti.graph.Edge;
import org.graffiti.graph.GraphElement;
import org.graffiti.graph.Node;
import org.graffiti.plugin.algorithm.AbstractAlgorithm;
import org.graffiti.plugin.algorithm.PreconditionException;
import org.graffiti.plugin.parameter.ObjectListParameter;
import org.graffiti.plugin.parameter.Parameter;
import org.graffiti.plugins.editcomponents.defaults.EdgeArrowShapeEditComponent;

import de.ipk_gatersleben.ag_nw.graffiti.GraphHelper;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.ConditionInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.GraphElementHelper;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.SampleInterface;
import de.ipk_gatersleben.ag_nw.graffiti.plugins.gui.editing_tools.script_helper.SubstanceInterface;
import de.ipk_gatersleben.ag_pbi.fluxbalance.startsim.FBAHelper;

/**
 * @author Eva
 */
public class StartSimulationAlgorithm extends AbstractAlgorithm {
	
	public StartSimulationAlgorithm(double minVal, double maxVal) {
		this.maxValue = maxVal;
		this.minValue = minVal;
	}
	
	private String currentSeries = "";
	private final double maxValue;
	private final double minValue;
	
	public String getName() {
		return "Viusalize steady state flux distribution";
	}
	
	@Override
	public String getCategory() {
		return "FBA";
	}
	
	@Override
	public String getDescription() {
		return "<html>" +
							"With this command you may interactively visualize the steady state flux distributions<br>" +
							"1. Load network<br>" +
							"2. Load flux data";
	}
	
	@Override
	public Parameter[] getParameters() {
		// pulldownmenue fuer SeriesData (P1,....P10);
		TreeSet<String> knownSeriesNames = getSeriesNames(getSelectedOrAllGraphElements());
		return new Parameter[] { new ObjectListParameter(knownSeriesNames.iterator().next(), "Simulationsparameter", "", knownSeriesNames) };
	}
	
	public static TreeSet<String> getSeriesNames(Collection<GraphElement> elements) {
		TreeSet<String> names = new TreeSet<String>();
		for (GraphElement ge : elements) {
			GraphElementHelper geh = new GraphElementHelper(ge);
			for (SubstanceInterface md : geh.getDataMappings()) {
				for (ConditionInterface sd : md) {
					
					String sdName = FBAHelper.getSeriesNameForComparison(sd);
					String sdName5;
					int index5 = sdName.lastIndexOf("/", sdName.length());
					
					if (index5 == -1) {
						sdName5 = sdName;
					} else {
						sdName5 = sdName.substring(0, index5);
					}
					
					// names.add(sd.getName());
					names.add(sdName5);
				}
			}
		}
		
		return names;
	}
	
	@Override
	public void setParameters(Parameter[] params) {
		int i = 0;
		// Uebernahme des im Pull-down Menues uebergebenen Serien-Namens
		currentSeries = (String) ((ObjectListParameter) params[i++]).getValue();
		// System.out.println("currentSeriesA: "+currentSeries);
	}
	
	@Override
	public void check() throws PreconditionException {
		if (graph == null)
			throw new PreconditionException("No active graph!");
	}
	
	public void execute() {
		
		double minVal = minValue;
		double maxVal = maxValue;
		double minimumTargetValueT = 1;
		
		double maximumTargetValueT = 30;
		double minimumTargetValueA = 5;
		double maximumTargetValueA = 36;
		
		// Gerste-Modell:
		// double maximumTargetValueT = 85;
		// double maximumTargetValueA= 145;
		// double maximumTargetValueT = 65;
		// double maximumTargetValueA= 105;
		
		long startO = System.currentTimeMillis();
		String checkOptSol = "optSol";
		
		try {
			// alle Methoden wwerden erst am ENde durchgefuehrt - Beschleunigung
			graph.getListenerManager().transactionStarted(this);
			
			// Graph grau einfaerben & Kantenbreite 1
			
			changeColorToGray();
			
			if (minVal < 0.0001) {
				minVal = 0;
			}
			
			checkOptSol = setEdgeThickness(minVal, maxVal, minimumTargetValueT,
								maximumTargetValueT, minimumTargetValueA,
								maximumTargetValueA);
			
			setColorForNodesWithMappingData();
			
		} catch (Exception e) {
			ErrorMsg.addErrorMessage(e);
		} finally {
			long curr1 = System.currentTimeMillis();
			graph.getListenerManager().transactionFinished(this, true);
			long curr2 = System.currentTimeMillis();
			System.out.println("Transaction Finished Time: " + (curr2 - curr1));
		}
		GraphHelper.issueCompleteRedrawForGraph(graph);
		long endO = System.currentTimeMillis();
		MainFrame.showMessage("Complete Processing Time: " + (endO - startO) + " ms", MessageType.INFO);
		
		
		//veraltet
		/*if (checkOptSol.equals("NoOptSol")) {
			
			// MainFrame.showMessage("Warning: No optimal solution found!", MessageType.INFO);
			//MainFrame.showMessageDialog("Warning: No optimal solution found!", "Information");
		}
		*/
		
	}
	
	private void setColorForNodesWithMappingData() {
		// Knoten mit gemappten Daten einfaerben
		for (GraphElement ge : graph.getGraphElements()) {
			if (ge instanceof Node) {
				Node n = (Node) ge;
				for (Edge e : n.getEdges()) {
					if (AttributeHelper.getOutlineColor(e).equals(Color.BLACK)) {
						AttributeHelper.setOutlineColor(n, Color.BLACK);
						AttributeHelper.setLabelColor(-1, n, Color.BLACK);
					}
				}
			}
		}
		// System.out.println("gemappte daten faerben: "+dauer);
	}
	
	// private void setEdgeThickness(double minValue, double maxValue,
	private String setEdgeThickness(double minValue, double maxValue,
						double minimumTargetValueT, double maximumTargetValueT,
						double minimumTargetValueA, double maximumTargetValueA) {
		String checkOptSol = "OptSol";
		String sdName1;
		for (GraphElement ge : graph.getEdges()) {
			// Setzen der relativen Kantendicke
			GraphElementHelper geh = new GraphElementHelper(ge);
			for (SubstanceInterface md : geh.getDataMappings()) {
				for (ConditionInterface sd : md) {
					for (SampleInterface sampleData : sd) {
						
						// Vorbereitung fuer korrekten sd.getNAme zu Vergleich mit currentseries
						String sdName = FBAHelper.getSeriesNameForComparison(sd);
						String sdName2;
						
						// NEU:
						// frueher: sdName = R1_3_3<*>P3
						// neu: sdName = R1_3_3<*>P3/unspecified
						// -> /unspecified loeschen
						
						int index2 = sdName.lastIndexOf("/", sdName.length());
						
						if (index2 == -1) {
							sdName2 = sdName;
						} else {
							sdName2 = sdName.substring(0, index2);
						}
						
						int index1 = sdName2.lastIndexOf("<*>", sdName2.length());
						if (index1 == -1) {
							sdName1 = sdName2;
						} else {
							sdName1 = sdName2.substring(index1 + 3, sdName2.length());
						}
						// System.out.println("sdName1 A: "+sdName1);
						
						String currentSeries2;
						
						int index3 = currentSeries.lastIndexOf("/", currentSeries.length());
						
						if (index3 == -1) {
							currentSeries2 = currentSeries;
						} else {
							currentSeries2 = currentSeries.substring(0, index3);
						}
						
						String sdName3;
						/*
						 * von Hendrik: Eva, dieser Knockout-Krams, in dem du "_R" suchst, hat zur Folge, dass
						 * es nicht mehr möglich ist, die Condition zu wählen. Sobald eine Condition "_R" enthält,
						 * dann gilt immer: sdName3.equals(currentSeries2) und damit wird die currentSeries
						 * ignoriert
						 */
						/*
						 * int index4 = sdName2.lastIndexOf("_R", sdName2.length());
						 * int index4 = sdName1.lastIndexOf("_R", sdName1.length());
						 * if(index4==-1){
						 * sdName3 = sdName1;
						 * }else{
						 * sdName3 = sdName1.substring(0, sdName1.length());
						 * //System.out.println("Neuer Name: "+sdName1);
						 * currentSeries2 = sdName3;
						 * }
						 */
						sdName3 = sdName1;
						
						// System.out.println("Neuer Name: "+sdName3);
						// System.out.println("currentSeries2: "+currentSeries2);
						
						if (sdName3.equals(currentSeries2)) {
							double val2 = sampleData.calcMean();
							double val3 = Math.abs(val2);
							String label = String.valueOf(val3);
							
							// Check if Optimal Solution found (val3 !=0)
							if (val3 == 0) {
								checkOptSol = "NoOptSol";
							}
							
							if (!Double.isNaN(val3)) {
								if (minValue == maxValue) {
									
									AttributeHelper.setArrowSize((Edge) ge, (maximumTargetValueA + minimumTargetValueA) / 2);
									AttributeHelper.setFrameThickNess(ge, (maximumTargetValueT + minimumTargetValueT) / 2);
									AttributeHelper.setOutlineColor(ge, Color.BLACK);
									AttributeHelper.setLabel((Edge) ge, label);
									AttributeHelper.setLabelColor(-1, ge, Color.RED);
								} else {
									
									double resT = (maximumTargetValueT - minimumTargetValueT) * (Math.abs(val3) - minValue) / (maxValue - minValue)
														+ minimumTargetValueT;
									double resA = (maximumTargetValueA - minimumTargetValueA) * (Math.abs(val3) - minValue) / (maxValue - minValue)
														+ minimumTargetValueA;
									AttributeHelper.setArrowSize((Edge) ge, resA);
									AttributeHelper.setFrameThickNess(ge, resT);
									AttributeHelper.setOutlineColor(ge, Color.BLACK);
									AttributeHelper.setLabel((Edge) ge, label);
									AttributeHelper.setLabelColor(-1, ge, Color.RED);
								}
							}
							
							// 1.Reversible edges suchen und in irreversible umwandeln bzw. shape = ----
							// 2. Negative Fluesse suchen und edges umdrehen
							if (ge instanceof Edge) {
								// 1.
								if (AttributeHelper.isSBMLreversibleReaction(ge)) {
									AttributeHelper.setArrowtail((Edge) ge, "");
								}
								// 2.
								if (val2 < 0) {
									AttributeHelper.setArrowhead((Edge) ge, "");
									AttributeHelper.setArrowtail((Edge) ge, EdgeArrowShapeEditComponent.standardArrow);
									AttributeHelper.setFillColor(ge, Color.BLACK);
								} else {
									AttributeHelper.setArrowhead((Edge) ge, EdgeArrowShapeEditComponent.standardArrow);
									AttributeHelper.setArrowtail((Edge) ge, "");
								}
							}
						}
						
					}
				}
			}
		}
		// System.out.println("rev, etc. : "+dauer);
		return checkOptSol;
	}
	
	private void changeColorToGray() {
		// System.out.println("Grau Start (at "+start+")");
		for (GraphElement ge : graph.getGraphElements()) {
			if (ge instanceof Edge) {
				AttributeHelper.setOutlineColor(ge, Color.GRAY);
				AttributeHelper.setArrowSize((Edge) ge, 1);
				AttributeHelper.setFrameThickNess(ge, 1);
				AttributeHelper.setLabel((Edge) ge, "0");
				AttributeHelper.setLabelColor(-1, ge, Color.GRAY);
			}
			if (ge instanceof Node) {
				Node n = (Node) ge;
				AttributeHelper.setOutlineColor(ge, Color.GRAY);
				AttributeHelper.setLabelColor(-1, n, Color.GRAY);
				AttributeHelper.setBorderWidth(ge, 1);
			}
		}
		// System.out.println("Grau: "+dauer);
	}
}
